import elementResizeDetectorMaker from 'element-resize-detector'
import globalUtils from '../utils/globalUtils'
import { nanoid } from 'nanoid'

let base = {
  data() {
    return {
      shadeStyle: {
        left: '0px',
        top: '0px',
        height: `${document.documentElement.clientHeight}px`,
        width: `${document.documentElement.clientWidth}px`,
      },
      erd: elementResizeDetectorMaker(),
      layerId: null,
    }
  },
  created() {
    this.layerId = nanoid()
    globalUtils.add(this.layerId, this)
  },
  destroyed() {
    globalUtils.remove(this.layerId)
    this.erd.uninstall(this.appendElement)
  },
  methods: {
    _dragBind(dragBox, moveBox) {
      const that = this
      dragBox.onmousedown = (e) => {
        const disX = e.clientX - moveBox.offsetLeft
        const disY = e.clientY - moveBox.offsetTop
        document.onmousemove = (e) => {
          e.preventDefault()
          let bodyWidth = parseFloat(that.shadeStyle.width)
          // that.appendElement !== null
          //   ? that.appendElement.offsetWidth
          //   : document.documentElement.clientWidth
          let bodyHeight = parseFloat(that.shadeStyle.height)
          // that.appendElement !== null
          //   ? that.appendElement.offsetHeight
          //   : document.documentElement.clientHeight
          let l = e.clientX - disX
          let t = e.clientY - disY
          const x = bodyWidth - moveBox.offsetWidth
          const y = bodyHeight - moveBox.offsetHeight
          l = l < 10 ? 0 : l > x - 10 ? x : l
          t = t < 10 ? 0 : t > y - 10 ? y : t
          moveBox.style.left = l < 0 ? 0 : l + 'px'
          moveBox.style.top = t < 0 ? 0 : t + 'px'
          return false
        }
        document.onmouseup = () => {
          document.onmousemove = null
          document.onmouseup = null
          return false
        }
        return false
      }
    },
    _boundaryDetect() {
      const that = this
      if (this.appendElement !== null) {
        that.erd.listenTo(this.appendElement, (element) => {
          that.$nextTick(() => {
            that.shadeStyle = {
              left: element.scrollLeft + 'px',
              top: element.scrollTop + 'px',
              height: element.offsetHeight + 'px',
              width: element.offsetWidth + 'px',
            }
            // that.resetLocation()
          })
        })
      } else {
        window.onresize = () => {
          //调用methods中的事件
          if (that.appendElement === null) {
            that.$nextTick(() => {
              that.shadeStyle = {
                left: '0px',
                top: '0px',
                height: `${document.documentElement.clientHeight}px`,
                width: `${document.documentElement.clientWidth}px`,
              }
              // that.resetLocation()
            })
          }
        }
      }
    },
    //自动居中
    autoCenter() {
      let left =
        parseFloat(this.shadeStyle.width) / 2 -
        Number(this.$refs.layer.offsetWidth) / 2
      let top =
        parseFloat(this.shadeStyle.height) / 2 -
        Number(this.$refs.layer.offsetHeight) / 2
      this.$refs.layer.style.top = top + 'px'
      this.$refs.layer.style.left = left + 'px'
    },
    resetLocation() {
      if (typeof this.location === 'string') {
        let columnCenter =
          parseFloat(this.shadeStyle.width) / 2 -
          Number(this.$refs.layer.offsetWidth) / 2
        let rowCenter =
          parseFloat(this.shadeStyle.height) / 2 -
          Number(this.$refs.layer.offsetHeight) / 2
        switch (this.location) {
          case 'lt':
            this.$refs.layer.style.top = '25px'
            this.$refs.layer.style.left = '25px'
            break
          case 'lc':
            this.$refs.layer.style.top = `${rowCenter}px`
            this.$refs.layer.style.left = '25px'
            break
          case 'lb':
            this.$refs.layer.style.bottom = '25px'
            this.$refs.layer.style.left = '25px'
            break
          case 'ct':
            this.$refs.layer.style.top = '25px'
            this.$refs.layer.style.left = `${columnCenter}px`
            break
          case 'cc':
            this.$refs.layer.style.top = `${rowCenter}px`
            this.$refs.layer.style.left = `${columnCenter}px`
            break
          case 'cb':
            this.$refs.layer.style.bottom = '25px'
            this.$refs.layer.style.left = `${columnCenter}px`
            break
          case 'rt':
            this.$refs.layer.style.top = '25px'
            this.$refs.layer.style.right = '25px'
            break
          case 'rc':
            this.$refs.layer.style.top = `${columnCenter}px`
            this.$refs.layer.style.right = '25px'
            break
          case 'rb':
            this.$refs.layer.style.bottom = '25px'
            this.$refs.layer.style.right = '25px'
            break
        }
      } else if (typeof this.location === 'object') {
        this.$refs.layer.style.top = this.location.top + 'px'
        this.$refs.layer.style.left = this.location.left + 'px'
      }
    },
  },
}
export default base
